/* Scrutari - Copyright (c) 2008-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.admin.htmlpages;

import fr.exemole.sctserver.api.SctEngine;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;
import net.mapeadores.util.html.HA;
import net.mapeadores.util.html.HtmlAttributes;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.text.LabelUtils;
import net.scrutari.data.BaseData;
import net.scrutari.data.CorpusData;
import net.scrutari.data.DataConstants;
import net.scrutari.data.FicheData;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.ScrutariDB;


/**
 *
 * @author Vincent Calame
 */
public class FichesPage extends AdminHtmlPage {

    HtmlAttributes baseAttributes = HA.classes("scrutari-list-Base");
    HtmlAttributes corpusAttributes = HA.classes("scrutari-list-Corpus");
    HtmlAttributes ficheAttributes = HA.classes("scrutari-list-Fiche");
    HtmlAttributes ulAttributes = HA.classes("scrutari-list-List").style("display: none;");
    private final ScrutariDB scrutariDB;

    public FichesPage(SctEngine engine, Lang workingLang) {
        super(engine, workingLang);
        this.scrutariDB = engine.getScrutariSession().getScrutariDB();
        addCssUrl("../resources/theme/css/list.css");
        addJsUrl("../resources/js/list.js");
    }

    @Override
    public void handleResponse(HttpServletResponse response) throws IOException {
        DataAccess dataAccess = scrutariDB.openDataAccess();
        initResponse(response);
        start();
        printTools();
        startBSContainer();
        printMessage();
        startBSRow();
        this
                .H2()
                .localize("_ title.sctconf.fiches")
                ._H2();
        int baseCount = dataAccess.getBaseCount();
        for (int i = 0; i < baseCount; i++) {
            BaseData baseData = dataAccess.getBaseDataByIndex(i);
            printBaseData(baseData, dataAccess);
        }
        endBSRow();
        endBSContainer();
        end();
        dataAccess.close();
    }

    private void printBaseData(BaseData baseData, DataAccess dataAccess) {
        int codebase = baseData.getCodeBase();
        int corpusCount = baseData.getCorpusCount();
        this
                .DIV(baseAttributes.attr("onclick", "affiche('ul_" + codebase + "');"))
                .escape(LabelUtils.seekLabelString(baseData.getPhrases(), DataConstants.BASE_LONGTITLE, workingLang, "?"))
                .space()
                .CODE()
                .escape('[')
                .append(corpusCount)
                .escape(']')
                ._CODE()
                ._DIV();
        this
                .UL(ulAttributes.id("ul_" + codebase));
        for (int i = 0; i < corpusCount; i++) {
            CorpusData corpusData = dataAccess.getCorpusDataByCode(baseData.getCodeCorpus(i));
            printCorpusData(corpusData, dataAccess);
        }
        this
                ._UL();
    }

    private int printCorpusData(CorpusData corpusData, DataAccess dataAccess) {
        int codecorpus = corpusData.getCodeCorpus();
        int ficheCount = corpusData.getFicheCount();
        String onClick = "affiche('ul_" + codecorpus + "');";
        this
                .LI(corpusAttributes.attr("onclick", onClick))
                .escape(LabelUtils.seekLabelString(corpusData.getPhrases(), DataConstants.CORPUS_TITLE, workingLang, "?"))
                .space()
                .CODE()
                .escape('[')
                .append(ficheCount)
                .escape(']')
                ._CODE()
                ._LI();
        this
                .UL(ulAttributes.attr("onclick", onClick).id("ul_" + codecorpus));
        for (int i = 0; i < ficheCount; i++) {
            FicheData ficheData = dataAccess.getFicheDataByCode(corpusData.getCodeFiche(i));
            printFicheData(ficheData);
        }
        this
                ._UL();
        return ficheCount;
    }

    private void printFicheData(FicheData ficheData) {
        this
                .LI(ficheAttributes)
                .escape(ficheData.getTitre())
                ._LI();
    }

}
