/* Scrutari - Copyright (c) 2008-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.admin.htmlpages;

import fr.exemole.sctserver.api.SctEngine;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;
import net.mapeadores.util.html.HA;
import net.mapeadores.util.html.HtmlAttributes;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.text.Label;
import net.mapeadores.util.text.LabelUtils;
import net.scrutari.data.BaseData;
import net.scrutari.data.DataConstants;
import net.scrutari.data.MotcleData;
import net.scrutari.data.ThesaurusData;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.ScrutariDB;


/**
 *
 * @author Vincent Calame
 */
public class MotsclesPage extends AdminHtmlPage {

    private final HtmlAttributes thesaurusCellAttributes = HA.classes("scrutari-list-Thesaurus");
    private final HtmlAttributes motcleTBodyAttributes = HA.style("display: none;");
    private final ScrutariDB scrutariDB;
    private final Lang[] motcleLangArray;

    public MotsclesPage(SctEngine engine, Lang workingLang) {
        super(engine, workingLang);
        scrutariDB = scrutariSession.getScrutariDB();
        addCssUrl("../resources/theme/css/list.css");
        addJsUrl("../resources/js/list.js");
        motcleLangArray = scrutariDB.getStats().getAllMotcleLangList().toArray(new Lang[0]);
    }

    @Override
    public void handleResponse(HttpServletResponse response) throws IOException {
        DataAccess dataAccess = scrutariDB.openDataAccess();
        initResponse(response);
        start();
        printTools();
        startBSContainer();
        printMessage();
        startBSRow();
        this
                .H2()
                .localize("_ title.sctconf.motscles")
                ._H2();
        this
                .P()
                .escape("ods : ")
                .A(HA.href("motscles.ods"))
                .escape("_all")
                ._A()
                ._P();
        this
                .TABLE(HA.classes("scrutari-list-Table"));
        this
                .THEAD();
        this
                .TR();
        for (Lang lang : motcleLangArray) {
            this
                    .TH()
                    .escape(lang)
                    ._TH();
        }
        this
                ._TR();
        this
                ._THEAD();
        thesaurusCellAttributes.colspan(motcleLangArray.length);
        HtmlAttributes baseCellAttr = HA.colspan(motcleLangArray.length);
        int baseCount = dataAccess.getBaseCount();
        for (int i = 0; i < baseCount; i++) {
            BaseData baseData = dataAccess.getBaseDataByIndex(i);
            printBaseData(baseData, dataAccess, baseCellAttr);
        }
        this
                ._TABLE();
        endBSRow();
        endBSContainer();
        end();
        dataAccess.close();
    }

    private void printBaseData(BaseData baseData, DataAccess dataAccess, HtmlAttributes baseCellAttr) {
        int thesaurusCount = baseData.getThesaurusCount();
        this
                .TBODY();
        this
                .TR();
        this
                .TD(baseCellAttr)
                .escape(LabelUtils.seekLabelString(baseData.getPhrases(), DataConstants.BASE_LONGTITLE, workingLang, "?"))
                .space()
                .CODE()
                .escape('[')
                .append(thesaurusCount)
                .escape(']')
                ._CODE()
                ._TD();
        this
                ._TR();
        this
                ._TBODY();
        for (int i = 0; i < thesaurusCount; i++) {
            ThesaurusData thesaurusData = dataAccess.getThesaurusDataByCode(baseData.getCodeThesaurus(i));
            printThesaurusData(thesaurusData, dataAccess);
        }
    }

    private void printThesaurusData(ThesaurusData thesaurusData, DataAccess dataAccess) {
        int codethesaurus = thesaurusData.getCodeThesaurus();
        int motcleCount = thesaurusData.getMotcleCount();
        this
                .TBODY(HA.attr("onclick", "afficheRowGroup('tbody_" + codethesaurus + "');"));
        this
                .TR();
        this
                .TD(thesaurusCellAttributes)
                .escape(LabelUtils.seekLabelString(thesaurusData.getPhrases(), DataConstants.THESAURUS_TITLE, workingLang, "?"))
                .space()
                .CODE()
                .escape('[')
                .append(motcleCount)
                .escape(']')
                ._CODE()
                ._TD();
        this
                ._TR();
        this
                ._TBODY();
        motcleTBodyAttributes.id("tbody_" + codethesaurus);
        this
                .TBODY(motcleTBodyAttributes);
        for (int i = 0; i < motcleCount; i++) {
            printMotcleData(dataAccess.getMotcleDataByCode(thesaurusData.getCodeMotcle(i)));
        }
        this
                ._TBODY();

    }

    private void printMotcleData(MotcleData motcleData) {
        this
                .TR();
        for (Lang lang : motcleLangArray) {
            this
                    .TD();
            Label libelle = motcleData.getLabelByLang(lang);
            if (libelle != null) {
                this
                        .escape(libelle.getLabelString());
            }
            this
                    ._TD();
        }
        this
                ._TR();
    }

}
