/* Scrutari - Copyright (c) 2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.admin.htmlpages.utils;

import fr.exemole.sctserver.servlets.admin.htmlpages.AdminHtmlPage;
import java.util.List;
import net.mapeadores.util.attr.AttributeKey;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.html.HA;
import net.scrutari.data.DataValidator;


/**
 *
 * @author Vincent Calame
 */
public final class ValidationPrint {

    private ValidationPrint() {

    }

    public static void printValidator(AdminHtmlPage hp) {
        DataValidator dataValidator = hp.getEngine().getEngineStorage().getConfDirectory().getDataValidator();
        boolean done = false;
        if (dataValidator != null) {
            List<Short> mandatoryList = dataValidator.getMandatoryFieldList();
            if (!mandatoryList.isEmpty()) {
                hp
                        .P()
                        .localize("_ label.sctconf.validation_mandatory")
                        .colon()
                        .space();
                append(hp, mandatoryList);
                hp
                        ._P();
                done = true;
            }
            List<Short> checkedList = dataValidator.getCheckedFieldList();
            if (!checkedList.isEmpty()) {
                hp
                        .P()
                        .localize("_ label.sctconf.validation_checked")
                        .colon()
                        .space();
                append(hp, checkedList);
                hp
                        ._P();
                done = true;
            }
            List<AttributeKey> uniqueList = dataValidator.getUniqueValueAttributeKeyList();
            if (!uniqueList.isEmpty()) {
                hp
                        .P()
                        .localize("_ label.sctconf.validation_unique")
                        .colon()
                        .space();
                appendAttributeKeyList(hp, uniqueList);
                hp
                        ._P();
                done = true;
            }
        }
        if (!done) {
            hp
                    .P()
                    .SPAN(HA.classes("scrutari-admin-Info"))
                    .localize("_ info.sctconf.none_validation")
                    ._SPAN()
                    ._P();
        }
    }

    private static void append(AdminHtmlPage hp, List<Short> list) {
        boolean next = false;
        for (Short field : list) {
            if (next) {
                hp.escape(", ");
            } else {
                next = true;
            }
            hp.localize(getLocKey(field));
        }
    }

    private static void appendAttributeKeyList(AdminHtmlPage hp, List<AttributeKey> list) {
        boolean next = false;
        for (AttributeKey attributeKey : list) {
            if (next) {
                hp.escape(", ");
            } else {
                next = true;
            }
            hp.escape(attributeKey.toString());
        }
    }

    private static String getLocKey(short field) {
        switch (field) {
            case DataValidator.GEOLOC_FIELD:
                return "_ label.field.geoloc";
            case DataValidator.SOUSTITRE_FIELD:
                return "_ label.field.soustitre";
            default:
                throw new SwitchException("Unknown field type: " + field);
        }
    }

}
