/* ScrutariLib_DB - Copyright (c) 2012-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.db.tools;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import net.mapeadores.util.attr.AttributeKey;
import net.scrutari.db.api.AttributeKeyCoder;
import net.scrutari.lexie.AlineaInteger;


/**
 *
 * @author Vincent Calame
 */
public class AttributeKeyCoderBuilder {

    private final Set<AttributeKey> existingSet = new HashSet<AttributeKey>();
    private final List<AttributeKey> primaryList = new ArrayList<AttributeKey>();
    private final List<AttributeKey> secondaryList = new ArrayList<AttributeKey>();
    private final List< AttributeKey> technicalList = new ArrayList<AttributeKey>();

    public AttributeKeyCoderBuilder() {
    }

    public boolean addPrimary(AttributeKey attributeKey) {
        if (existingSet.contains(attributeKey)) {
            return false;
        }
        if (primaryList.size() < 999) {
            primaryList.add(attributeKey);
        } else {
            technicalList.add(attributeKey);
        }
        existingSet.add(attributeKey);
        return true;
    }

    public boolean addSecondary(AttributeKey attributeKey) {
        if (existingSet.contains(attributeKey)) {
            return false;
        }
        if (secondaryList.size() < 999) {
            secondaryList.add(attributeKey);
        } else {
            technicalList.add(attributeKey);
        }
        existingSet.add(attributeKey);
        return true;
    }

    public boolean addTechnical(AttributeKey attributeKey) {
        if (existingSet.contains(attributeKey)) {
            return false;
        }
        technicalList.add(attributeKey);
        existingSet.add(attributeKey);
        return true;
    }

    public AttributeKeyCoder toAttributeKeyCoder() {
        Map<AttributeKey, Integer> alineaRootCodeMap = new HashMap<AttributeKey, Integer>();
        int primaryMax = AlineaInteger.PRIMARY_ATTRIBUTEGROUP_START;
        for (AttributeKey attributeDef : primaryList) {
            alineaRootCodeMap.put(attributeDef, primaryMax);
            primaryMax++;
        }
        int secondaryMax = AlineaInteger.SECONDARY_ATTRIBUTEGROUP_START;
        for (AttributeKey attributeDef : secondaryList) {
            alineaRootCodeMap.put(attributeDef, secondaryMax);
            secondaryMax++;
        }
        Integer itg = AttributeKeyCoder.TECHNICAL_ATTRIBUTE;
        for (AttributeKey attributeDef : technicalList) {
            alineaRootCodeMap.put(attributeDef, itg);
        }
        return new InternalAttributeKeyCoder(alineaRootCodeMap);
    }


    private static class InternalAttributeKeyCoder implements AttributeKeyCoder {

        private final Map<AttributeKey, Integer> alineaRootCodeMap;

        private InternalAttributeKeyCoder(Map<AttributeKey, Integer> alineaRootCodeMap) {
            this.alineaRootCodeMap = alineaRootCodeMap;
        }

        @Override
        public int getAlineaRootCode(AttributeKey attributeKey) {
            Integer itg = alineaRootCodeMap.get(attributeKey);
            if (itg == null) {
                return AttributeKeyCoder.UNKNOWN_ATTRIBUTE;
            }
            return itg;
        }

    }

}
