/* ScrutariLib_DB - Copyright (c) 2005-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.db.tools.util;

import net.mapeadores.util.localisation.Lang;
import net.scrutari.data.BaseData;
import net.scrutari.data.CorpusData;
import net.scrutari.data.FicheData;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.stats.FicheStats;
import net.scrutari.db.api.stats.ScrutariDBStats;


/**
 * Propose des méthodes statitiques utilitaires pour récupérer des Datarmations
 * sur les bases.
 *
 * @author Vincent Calame
 */
public class DataUtils {

    static final char SEUIL1 = '\u0080';
    static final char SEUIL2 = '\u0800';

    /**
     * Retourne le nombre de caractères supplémentaires introduit par le codage
     * UTF8 du caractère en argument. Par exemple, la méthode retournera 0 pour
     * le caractère « a » qui en UTF-8 s'écrit « a » et elle retournera 1 pour
     * le caractère « é » qui en UTF-8 s'écrit « Ã© ».
     */
    public static int getDecalageUTF8(char c) {
        if (c < SEUIL1) {
            return 0;
        }
        if (c < SEUIL2) {
            return 1;
        }
        return 2;
    }

    private DataUtils() {
    }

    public static String getHref(FicheData ficheData, DataAccess dataAccess, Lang lang) {
        BaseData baseData = dataAccess.getBaseDataByCode(dataAccess.getCorpusDataByCode(ficheData.getCodeCorpus()).getCodeBase());
        Lang checkedLang = baseData.checkAvailableLang(lang);
        return ficheData.getHref(checkedLang);
    }

    public static int getFicheCountByCorpus(DataAccess dataAccess, int[] codecorpusArray, Lang[] langArray, ScrutariDBStats scrutariDBStats) {
        int resultat = 0;
        if (langArray != null) {
            int corpusCount = codecorpusArray.length;
            for (int i = 0; i < corpusCount; i++) {
                int codecorpus = codecorpusArray[i];
                FicheStats ficheStats = scrutariDBStats.getFicheStats(codecorpus);
                int langCount = ficheStats.getLangCount();
                for (int k = 0; k < langCount; k++) {
                    Lang lang = ficheStats.getLang(k);
                    if (contains(langArray, lang)) {
                        resultat = resultat + ficheStats.getFicheCount(k);
                    }
                }
            }
        } else {
            int corpusCount = codecorpusArray.length;
            for (int i = 0; i < corpusCount; i++) {
                CorpusData corpusData = dataAccess.getCorpusDataByCode(codecorpusArray[i]);
                if (corpusData != null) {
                    resultat = resultat + corpusData.getFicheCount();
                }
            }
        }
        return resultat;
    }

    private static boolean contains(Lang[] array, Lang value) {
        int length = array.length;
        for (int i = 0; i < length; i++) {
            if (array[i].equals(value)) {
                return true;
            }
        }
        return false;
    }

}
