/* ScrutariLib_SearchEngine - Copyright (c) 2006-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools.options;

import java.util.AbstractList;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.RandomAccess;
import java.util.Set;
import net.mapeadores.util.localisation.Lang;
import net.scrutari.data.CorpusData;
import net.scrutari.data.ThesaurusData;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.codes.Codes;
import net.scrutari.db.api.stats.ScrutariDBStats;
import net.scrutari.db.tools.codes.CodesUtils;
import net.scrutari.searchengine.api.options.Category;
import net.scrutari.searchengine.api.options.SearchOptions;
import net.scrutari.searchengine.api.options.SearchOptionsDef;
import net.scrutari.searchengine.tools.EligibilityBuffer;


/**
 *
 * @author Vincent Calame
 */
public final class OptionsUtils {

    public final static List<Category> EMPTY_CATEGORYLIST = Collections.emptyList();

    private OptionsUtils() {
    }

    public static Codes getSelectionThesaurusCodeList(ScrutariDBStats scrutariDBStats, DataAccess dataAccess, Codes corpusCodes) {
        if (corpusCodes == null) {
            return null;
        }
        Set<Integer> thesaurusSet = new LinkedHashSet<Integer>();
        int corpusCount = corpusCodes.getCodeCount();
        Codes thesaurusCodeList = dataAccess.getThesaurusCodes();
        int thesaurusCount = thesaurusCodeList.getCodeCount();
        for (int i = 0; i < corpusCount; i++) {
            int codecorpus = corpusCodes.getCode(i);
            for (int j = 0; j < thesaurusCount; j++) {
                int codethesaurus = thesaurusCodeList.getCode(j);
                if (scrutariDBStats.hasIndexation(codecorpus, codethesaurus)) {
                    thesaurusSet.add(codethesaurus);
                }
            }
        }
        return CodesUtils.toCodes(thesaurusSet);
    }

    public static Codes getFicheCodeList(DataAccess dataAccess, SearchOptions searchOptions) {
        EligibilityBuffer eligibilityBuffer = new EligibilityBuffer(dataAccess, searchOptions.getFicheEligibility());
        SearchOptionsDef searchOptionsDef = searchOptions.getSearchOptionsDef();
        Set<Lang> langSet = null;
        List<Lang> langList = searchOptionsDef.getFilterLangList();
        if (!langList.isEmpty()) {
            langSet = new HashSet<Lang>(langList);
        }
        Codes corpusCodes = searchOptions.getCorpusCodes();
        if (corpusCodes == null) {
            corpusCodes = dataAccess.getCorpusCodes();
        }
        List<Integer> list = new ArrayList<Integer>();
        int corpusCount = corpusCodes.getCodeCount();
        for (int i = 0; i < corpusCount; i++) {
            int codecorpus = corpusCodes.getCode(i);
            CorpusData corpusData = dataAccess.getCorpusDataByCode(codecorpus);
            int ficheCount = corpusData.getFicheCount();
            for (int j = 0; j < ficheCount; j++) {
                int codefiche = corpusData.getCodeFiche(j);
                boolean accept = true;
                if (langSet != null) {
                    Lang lang = dataAccess.getFicheLang(codefiche);
                    if (!langSet.contains(lang)) {
                        accept = false;
                    }
                }
                if ((accept) && (!eligibilityBuffer.acceptFiche(codefiche))) {
                    accept = false;
                }
                if (accept) {
                    list.add(codefiche);
                }
            }
        }
        return CodesUtils.toCodes(list);
    }

    public static Codes getMotcleCodeList(DataAccess dataAccess, SearchOptions searchOptions) {
        EligibilityBuffer eligibilityBuffer = new EligibilityBuffer(dataAccess, searchOptions.getFicheEligibility());
        Codes thesaurusCodes = searchOptions.getThesaurusCodes();
        if (thesaurusCodes == null) {
            thesaurusCodes = dataAccess.getThesaurusCodes();
        }
        List<Integer> list = new ArrayList<Integer>();
        int thesaurusCount = thesaurusCodes.getCodeCount();
        for (int i = 0; i < thesaurusCount; i++) {
            int codethesaurus = thesaurusCodes.getCode(i);
            ThesaurusData thesaurusData = dataAccess.getThesaurusDataByCode(codethesaurus);
            int motcleCount = thesaurusData.getMotcleCount();
            for (int j = 0; j < motcleCount; j++) {
                int codemotcle = thesaurusData.getCodeMotcle(j);
                if (eligibilityBuffer.acceptMotcle(codemotcle)) {
                    list.add(codemotcle);
                }
            }
        }
        return CodesUtils.toCodes(list);
    }

    public static List<Category> wrap(Category[] array) {
        return new CategoryList(array);
    }


    private static class CategoryList extends AbstractList<Category> implements RandomAccess {

        private final Category[] array;

        private CategoryList(Category[] array) {
            this.array = array;
        }

        @Override
        public int size() {
            return array.length;
        }

        @Override
        public Category get(int index) {
            return array[index];
        }

    }

}
