/* SctServer - Copyright (c) 2016-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.impl;

import fr.exemole.sctserver.api.EngineMetadata;
import java.net.URL;
import net.mapeadores.util.attr.Attribute;
import net.mapeadores.util.attr.AttributeKey;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.text.Label;
import net.mapeadores.util.text.LabelUtils;
import net.mapeadores.util.text.Phrases;


/**
 *
 * @author Vincent Calame
 */
class EngineMetadataImpl implements EngineMetadata {

    private EngineMetadata source;

    EngineMetadataImpl() {

    }

    void reload(EngineMetadata source) {
        this.source = source;
    }

    @Override
    public int getLabelCount() {
        if (source != null) {
            return source.getLabelCount();
        } else {
            return 0;
        }
    }

    @Override
    public Label getLabelByIndex(int i) {
        if (source != null) {
            return source.getLabelByIndex(i);
        } else {
            throw new IndexOutOfBoundsException();
        }
    }

    @Override
    public Label getLabelByLang(Lang lang) {
        if (source != null) {
            return source.getLabelByLang(lang);
        } else {
            return null;
        }
    }

    @Override
    public Phrases getPhrases() {
        if (source != null) {
            return source.getPhrases();
        } else {
            return LabelUtils.EMPTY_PHRASES;
        }
    }

    @Override
    public URL getIcon() {
        if (source != null) {
            return source.getIcon();
        } else {
            return null;
        }
    }

    @Override
    public URL getWebsite() {
        if (source != null) {
            return source.getWebsite();
        } else {
            return null;
        }
    }

    @Override
    public Lang getDefaultLang() {
        if (source != null) {
            return source.getDefaultLang();
        } else {
            return null;
        }
    }

    @Override
    public Attribute getAttributeByKey(AttributeKey key) {
        if (source != null) {
            return source.getAttributeByKey(key);
        } else {
            return null;
        }
    }

    @Override
    public int getAttributeCount() {
        if (source != null) {
            return source.getAttributeCount();
        } else {
            return 0;
        }
    }

    @Override
    public Attribute getAttributeByIndex(int i) {
        if (source != null) {
            return source.getAttributeByIndex(i);
        } else {
            throw new IndexOutOfBoundsException();
        }
    }

}
