/* SctServer - Copyright (c) 2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.impl;

import fr.exemole.sctserver.api.FieldVariantManager;
import fr.exemole.sctserver.api.fieldvariant.FieldVariant;
import fr.exemole.sctserver.tools.fieldvariant.FieldVariantBuilder;
import fr.exemole.sctserver.tools.fieldvariant.FieldVariantUtils;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 *
 * @author Vincent Calame
 */
class FieldVariantManagerImpl implements FieldVariantManager {

    private final static FieldVariant EMPTY_VARIANT_INSTANCE;
    private final static FieldVariant QUERY_VARIANT_INSTANCE;
    private final static FieldVariant DATA_VARIANT_INSTANCE;
    private final static FieldVariant GEO_VARIANT_INSTANCE;
    private Map<String, FieldVariant> fieldVariantMap = new HashMap<String, FieldVariant>();
    private List<FieldVariant> fieldVariantList = FieldVariantUtils.EMPTY_FIELDWARIANTLIST;

    static {
        FieldVariantBuilder emptyBuilder = new FieldVariantBuilder(EMPTY_VARIANT);
        EMPTY_VARIANT_INSTANCE = emptyBuilder.toFieldVariant();
        FieldVariantBuilder queryBuilder = new FieldVariantBuilder(QUERY_VARIANT);
        queryBuilder.parseFicheFields("codecorpus,mtitre,msoustitre,mattrs_all,attrs,mcomplements,year,href,icon");
        queryBuilder.parseMotcleFields("mlabels");
        QUERY_VARIANT_INSTANCE = queryBuilder.toFieldVariant();
        FieldVariantBuilder dataBuilder = new FieldVariantBuilder(DATA_VARIANT);
        dataBuilder.parseFicheFields("codecorpus,titre,soustitre,complements,year,href,icon");
        dataBuilder.parseMotcleFields("labels");
        DATA_VARIANT_INSTANCE = dataBuilder.toFieldVariant();
        FieldVariantBuilder geoBuilder = new FieldVariantBuilder(GEO_VARIANT);
        geoBuilder.parseFicheFields("-codefiche");
        geoBuilder.getAliasBuilder("name").addCoreField(FieldVariant.FICHE_TITRE);
        geoBuilder.getAliasBuilder("url").addCoreField(FieldVariant.FICHE_HREF);
        geoBuilder.parseMotcleFields("labels");
        GEO_VARIANT_INSTANCE = geoBuilder.toFieldVariant();
    }

    FieldVariantManagerImpl() {
        testDefault(fieldVariantMap);
    }

    @Override
    public FieldVariant getFieldVariant(String name) {
        return fieldVariantMap.get(name);
    }

    @Override
    public List<FieldVariant> getFieldVariantList() {
        return fieldVariantList;
    }

    void reload(Map<String, FieldVariant> map) {
        if (map == null) {
            Map<String, FieldVariant> newMap = new HashMap<String, FieldVariant>();
            testDefault(newMap);
            this.fieldVariantMap = newMap;
            this.fieldVariantList = FieldVariantUtils.EMPTY_FIELDWARIANTLIST;
        } else {
            Map<String, FieldVariant> newMap = new HashMap<String, FieldVariant>(map);
            testDefault(newMap);
            this.fieldVariantMap = newMap;
            int size = map.size();
            if (size == 0) {
                this.fieldVariantList = FieldVariantUtils.EMPTY_FIELDWARIANTLIST;
            } else {
                this.fieldVariantList = FieldVariantUtils.wrap(map.values().toArray(new FieldVariant[size]));
            }
        }
    }

    private static void testDefault(Map<String, FieldVariant> fieldVariantMap) {
        testDefault(fieldVariantMap, EMPTY_VARIANT, EMPTY_VARIANT_INSTANCE);
        testDefault(fieldVariantMap, QUERY_VARIANT, QUERY_VARIANT_INSTANCE);
        testDefault(fieldVariantMap, DATA_VARIANT, DATA_VARIANT_INSTANCE);
        testDefault(fieldVariantMap, GEO_VARIANT, GEO_VARIANT_INSTANCE);
    }

    private static void testDefault(Map<String, FieldVariant> fieldVariantMap, String name, FieldVariant defaultVariant) {
        if (!fieldVariantMap.containsKey(name)) {
            fieldVariantMap.put(name, defaultVariant);
        }
    }

}
