/* SctServer - Copyright (c) 2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.json.attributedef;

import fr.exemole.sctserver.api.AttributeDefManager;
import java.io.IOException;
import java.util.List;
import net.mapeadores.util.attr.AttributeDef;
import net.mapeadores.util.attr.AttributeKey;
import net.mapeadores.util.json.JSONWriter;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.text.Label;
import net.mapeadores.util.text.LabelUtils;
import net.scrutari.data.ScrutariAttributes;


/**
 *
 * @author Vincent Calame
 */
public final class AttributeDefJson_A {

    private AttributeDefJson_A() {

    }

    public static void properties(JSONWriter jsonWriter, AttributeDefManager attributeDefManager, Lang lang) throws IOException {
        jsonWriter.key("attributes");
        jsonWriter.object();
        addAttributeDefList(jsonWriter, attributeDefManager, AttributeDefManager.PRIMARY_GROUP, lang);
        addAttributeDefList(jsonWriter, attributeDefManager, AttributeDefManager.SECONDARY_GROUP, lang);
        addAttributeDefList(jsonWriter, attributeDefManager, AttributeDefManager.TECHNICAL_GROUP, lang);
        jsonWriter.endObject();
    }

    private static void addAttributeDefList(JSONWriter jsonWriter, AttributeDefManager attributeDefManager, String name, Lang lang) throws IOException {
        List<AttributeDef> attributeDefList = attributeDefManager.getAttributeDefList(name);
        jsonWriter.key(name);
        jsonWriter.array();
        for (AttributeDef attributeDef : attributeDefList) {
            jsonWriter.object();
            AttributeKey attributeKey = attributeDef.getAttributeKey();
            jsonWriter.key("key");
            jsonWriter.value(attributeKey.toString());
            String type = ScrutariAttributes.getFormatType(attributeDef);
            jsonWriter.key("type");
            jsonWriter.value(type);
            Label label = LabelUtils.getLangPartCheckedLabel(attributeDef, lang);
            if (label == null) {
                label = LabelUtils.getFirstLabel(attributeDef);
            }
            jsonWriter.key("lib");
            if (label != null) {
                jsonWriter.value(label.getLabelString());
            } else {
                jsonWriter.value(attributeDef.getAttributeKey().toString());
            }
            jsonWriter.endObject();
        }
        jsonWriter.endArray();
    }

}
