/* SctServer - Copyright (c) 2015-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.json.fichedata;

import fr.exemole.sctserver.api.AttributeDefManager;
import fr.exemole.sctserver.api.fieldvariant.FieldVariant;
import fr.exemole.sctserver.json.JsonUtils;
import java.io.IOException;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import net.mapeadores.util.attr.Attribute;
import net.mapeadores.util.attr.AttributeDef;
import net.mapeadores.util.attr.AttributeHolder;
import net.mapeadores.util.attr.AttributeKey;
import net.mapeadores.util.json.CommonJson;
import net.mapeadores.util.json.JSONWriter;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.primitives.FuzzyDate;
import net.scrutari.data.FicheData;
import net.scrutari.data.ScrutariAttributes;
import net.scrutari.datauri.BaseURI;
import net.scrutari.datauri.FicheURI;
import net.scrutari.db.api.AttributeKeyCoder;
import net.scrutari.db.tools.util.BaseCheck;
import net.scrutari.lexie.AlineaInteger;
import net.scrutari.searchengine.api.result.FicheSearchResultInfo;
import net.scrutari.searchengine.api.result.InAlineaOccurrences;
import net.scrutari.searchengine.api.result.SearchTokenOccurrence;


/**
 *
 * @author Vincent Calame
 */
public final class FicheDataJson_A {


    private FicheDataJson_A() {

    }

    public static void object(JSONWriter jsonWriter, FicheData ficheData, FieldVariant fieldVariant, BaseCheck baseCheck, FicheSearchResultInfo info, AttributeDefManager attributeDefManager, AttributeKeyCoder attributeKeyCoder) throws IOException {
        jsonWriter.object();
        properties(jsonWriter, ficheData, fieldVariant, baseCheck, info, attributeDefManager, attributeKeyCoder);
        jsonWriter.endObject();
    }

    public static void properties(JSONWriter jsonWriter, FicheData ficheData, FieldVariant fieldVariant, BaseCheck baseCheck, FicheSearchResultInfo info, AttributeDefManager attributeDefManager, AttributeKeyCoder attributeKeyCoder) throws IOException {
        if (fieldVariant.isFicheWith(FieldVariant.FICHE_CODEFICHE)) {
            jsonWriter.key("codefiche");
            jsonWriter.value(ficheData.getCodeFiche());
        }
        if (fieldVariant.isFicheWith(FieldVariant.FICHE_CODECORPUS)) {
            jsonWriter.key("codecorpus");
            jsonWriter.value(ficheData.getCodeCorpus());
        }
        if (fieldVariant.isFicheWith(FieldVariant.FICHE_CODEBASE)) {
            jsonWriter.key("codebase");
            jsonWriter.value(ficheData.getCodeBase());
        }
        if (fieldVariant.isFicheWith(FieldVariant.FICHE_BASENAME)) {
            BaseURI baseURI = ficheData.getFicheURI().getCorpusURI().getBaseURI();
            jsonWriter.key("authority");
            jsonWriter.value(baseURI.getAuthority());
            jsonWriter.key("basename");
            jsonWriter.value(baseURI.getBaseName());
        }
        if (fieldVariant.isFicheWith(FieldVariant.FICHE_CORPUSNAME)) {
            FicheURI ficheURI = ficheData.getFicheURI();
            jsonWriter.key("corpusname");
            jsonWriter.value(ficheURI.getCorpusURI().getCorpusName());
        }
        if (fieldVariant.isFicheWith(FieldVariant.FICHE_FICHEID)) {
            FicheURI ficheURI = ficheData.getFicheURI();
            jsonWriter.key("ficheid");
            jsonWriter.value(ficheURI.getFicheId());
        }
        if (fieldVariant.isFicheWith(FieldVariant.FICHE_TITRE)) {
            String titre = ficheData.getTitre();
            if (titre != null) {
                jsonWriter.key("titre");
                jsonWriter.value(ficheData.getTitre());
            }
        }
        if (fieldVariant.isFicheWith(FieldVariant.FICHE_SOUSTITRE)) {
            String soustitre = ficheData.getSoustitre();
            if (soustitre != null) {
                jsonWriter.key("soustitre");
                jsonWriter.value(soustitre);
            }
        }
        if (fieldVariant.isFicheWith(FieldVariant.FICHE_HREF)) {
            String href = ficheData.getHref(baseCheck.getCheckedLang());
            if (href != null) {
                jsonWriter.key("href");
                jsonWriter.value(href);
            }
        }
        if (fieldVariant.isFicheWith(FieldVariant.FICHE_LANG)) {
            Lang lang = ficheData.getLang();
            if (!lang.equals(FicheData.UNDETERMINED_LANG)) {
                jsonWriter.key("lang");
                jsonWriter.value(lang.toString());
            }
        }
        if (fieldVariant.isFicheWith(FieldVariant.FICHE_YEAR)) {
            FuzzyDate date = ficheData.getDate();
            if (date != null) {
                jsonWriter.key("annee");
                jsonWriter.value(date.getYear());
            }
        }
        if (fieldVariant.isFicheWith(FieldVariant.FICHE_DATE)) {
            FuzzyDate date = ficheData.getDate();
            if (date != null) {
                jsonWriter.key("date");
                jsonWriter.value(date.toString());
            }
        }
        if (fieldVariant.isFicheWith(FieldVariant.FICHE_FICHEICON)) {
            String ficheIcon = ficheData.getFicheIcon();
            if (ficheIcon != null) {
                jsonWriter.key("ficheicon");
                jsonWriter.value(ficheIcon);
            }
        }
        if (fieldVariant.isFicheWith(FieldVariant.FICHE_ICON)) {
            String icon = ficheData.getFicheIcon();
            if (icon == null) {
                icon = baseCheck.getBaseIcon();
            }
            if (icon != null) {
                jsonWriter.key("icon");
                jsonWriter.value(icon);
            }
        }
        if (fieldVariant.isFicheWith(FieldVariant.FICHE_GEO) && ficheData.isWithGeo()) {
            jsonWriter.key("lat");
            jsonWriter.value(ficheData.getLatitude().toString());
            jsonWriter.key("lon");
            jsonWriter.value(ficheData.getLongitude().toString());
        }
        if (fieldVariant.isFicheWith(FieldVariant.FICHE_COMPLEMENTS)) {
            int maxNumber = ficheData.getComplementMaxNumber();
            if (maxNumber > 0) {
                boolean inclus = false;
                for (int num = 1; num <= maxNumber; num++) {
                    String comp = ficheData.getComplementByNumber(num);
                    if (comp == null) {
                        continue;
                    }
                    if (!inclus) {
                        jsonWriter.key("complementArray");
                        jsonWriter.array();
                        inclus = true;
                    }
                    jsonWriter.object();
                    jsonWriter.key("num");
                    jsonWriter.value(num);
                    jsonWriter.key("comp");
                    jsonWriter.value(comp);
                    jsonWriter.endObject();
                }
                if (inclus) {
                    jsonWriter.endArray();
                }
            }
        }
        if (fieldVariant.isFicheWith(FieldVariant.FICHE_ATTRIBUTES)) {
            short attrType = fieldVariant.getFicheWithType(FieldVariant.FICHE_ATTRIBUTES);
            jsonWriter.key("attrMap");
            if (attrType == FieldVariant.ALL_TYPE) {
                CommonJson.object(jsonWriter, ficheData.getAttributHolder());
            } else {
                jsonWriter.object();
                AttributeHolder attributeHolder = ficheData.getAttributHolder();
                int attributeCount = attributeHolder.getAttributeCount();
                for (int i = 0; i < attributeCount; i++) {
                    Attribute attribute = attributeHolder.getAttributeByIndex(i);
                    AttributeKey attributeKey = attribute.getAttributeKey();
                    int alineaRootCode = attributeKeyCoder.getAlineaRootCode(attributeKey);
                    boolean include = false;
                    if (alineaRootCode == AttributeKeyCoder.TECHNICAL_ATTRIBUTE) {
                        include = true;
                    } else if ((attrType == FieldVariant.PRIMARY_TYPE) && (AlineaInteger.getAttributeCategory(alineaRootCode) == AlineaInteger.PRIMARYATTRIBUTE_CATEGORY)) {
                        include = true;
                    }
                    if (include) {
                        jsonWriter.key(attributeKey.toString());
                        int valCount = attribute.getValueCount();
                        jsonWriter.array();
                        for (int j = 0; j < valCount; j++) {
                            jsonWriter.value(attribute.getValue(j));
                        }
                        jsonWriter.endArray();
                    }
                }
                jsonWriter.endObject();
            }
        }
        List<FieldVariant.Alias> aliasList = fieldVariant.getFicheAliasList();
        for (FieldVariant.Alias alias : aliasList) {
            String aliasValue = JsonUtils.getAliasValue(alias, ficheData, baseCheck.getCheckedLang());
            if (aliasValue.length() > 0) {
                jsonWriter.key(alias.getName());
                jsonWriter.value(aliasValue);
            }
        }
        if (info == null) {
            return;
        }
        Map<Integer, InAlineaOccurrences> map = new HashMap<Integer, InAlineaOccurrences>();
        Set<Integer> alineaRootCodeSet = null;
        List<InAlineaOccurrences> occurrencesList = info.getInAlineaOccurrencesList();
        int occurrencesLength = occurrencesList.size();
        for (int i = 0; i < occurrencesLength; i++) {
            InAlineaOccurrences occurrences = occurrencesList.get(i);
            int alineaInteger = occurrences.getAlineaInteger();
            map.put(alineaInteger, occurrences);
            if (AlineaInteger.isAttributeAlineaInteger(alineaInteger)) {
                int alineaRootCode = AlineaInteger.toAlineaRootCode(alineaInteger);
                if (alineaRootCodeSet == null) {
                    alineaRootCodeSet = new HashSet<Integer>();
                }
                alineaRootCodeSet.add(alineaRootCode);
            }
        }
        if (fieldVariant.isFicheWith(FieldVariant.FICHE_MTITRE)) {
            String titre = ficheData.getTitre();
            if (titre != null) {
                jsonWriter.key("mtitre");
                JsonUtils.writeMValue(jsonWriter, titre, JsonUtils.getList(map, AlineaInteger.TITRE));
            }
        }
        if (fieldVariant.isFicheWith(FieldVariant.FICHE_MSOUSTITRE)) {
            String soustitre = ficheData.getSoustitre();
            if (soustitre != null) {
                jsonWriter.key("msoustitre");
                JsonUtils.writeMValue(jsonWriter, soustitre, JsonUtils.getList(map, AlineaInteger.SOUSTITRE));
            }
        }
        if (fieldVariant.isFicheWith(FieldVariant.FICHE_MCOMPLEMENTS)) {
            boolean all = (fieldVariant.getFicheWithType(FieldVariant.FICHE_MCOMPLEMENTS) == FieldVariant.ALL_TYPE);
            int maxNumber = ficheData.getComplementMaxNumber();
            if (maxNumber > 0) {
                boolean started = false;
                for (int num = 1; num <= maxNumber; num++) {
                    String comp = ficheData.getComplementByNumber(num);
                    if (comp == null) {
                        continue;
                    }
                    List<SearchTokenOccurrence> searchTokenOccurrenceList = JsonUtils.getList(map, AlineaInteger.getComplementAlineaInteger(num));
                    if ((searchTokenOccurrenceList == null) && (!all)) {
                        continue;
                    }
                    if (!started) {
                        jsonWriter.key("mcomplementArray");
                        jsonWriter.array();
                        started = true;
                    }
                    jsonWriter.object();
                    jsonWriter.key("num");
                    jsonWriter.value(num);
                    jsonWriter.key("mcomp");
                    JsonUtils.writeMValue(jsonWriter, comp, searchTokenOccurrenceList);
                    jsonWriter.endObject();
                }
                if (started) {
                    jsonWriter.endArray();
                }
            }
        }
        if (fieldVariant.isFicheWith(FieldVariant.FICHE_MATTRIBUTES)) {
            short type = fieldVariant.getFicheWithType(FieldVariant.FICHE_MATTRIBUTES);
            boolean onlyConcerned = (type == FieldVariant.CONCERNED_TYPE);
            boolean onlyPrimary = (type == FieldVariant.PRIMARY_TYPE);
            AttributeHolder attributeHolder = ficheData.getAttributHolder();
            int attributeCount = attributeHolder.getAttributeCount();
            boolean started = false;
            for (int i = 0; i < attributeCount; i++) {
                Attribute attribute = attributeHolder.getAttributeByIndex(i);
                AttributeKey attributeKey = attribute.getAttributeKey();
                int alineaRootCode = attributeKeyCoder.getAlineaRootCode(attributeKey);
                if (alineaRootCode < 1) {
                    continue;
                }
                AttributeDef attributeDef = attributeDefManager.getAttributeDef(attributeKey);
                boolean isNotConcerned = ((alineaRootCodeSet == null) || (!alineaRootCodeSet.contains(alineaRootCode)));
                boolean isBlock = ScrutariAttributes.isBlock(attributeDef);
                if ((isBlock) && (isNotConcerned)) {
                    continue;
                }
                boolean stop = false;
                if (onlyConcerned) {
                    if (isNotConcerned) {
                        stop = true;
                    }
                } else if (onlyPrimary) {
                    if (!ScrutariAttributes.ownsTo(attributeDef, AttributeDefManager.PRIMARY_GROUP)) {
                        if (isNotConcerned) {
                            stop = true;
                        }
                    }
                }
                if (stop) {
                    continue;
                }
                if (!started) {
                    jsonWriter.key("mattrMap");
                    jsonWriter.object();
                    started = true;
                }
                jsonWriter.key(attributeKey.toString());
                int valCount = attribute.getValueCount();
                jsonWriter.array();
                for (int j = 0; j < valCount; j++) {
                    String value = attribute.getValue(j);
                    int alineaInteger = AlineaInteger.getValueAlineaInteger(alineaRootCode, j);
                    List<SearchTokenOccurrence> searchTokenOccurrenceList = JsonUtils.getList(map, alineaInteger);
                    if ((searchTokenOccurrenceList != null) || (!isBlock)) {
                        JsonUtils.writeMValue(jsonWriter, value, searchTokenOccurrenceList);
                    }
                }
                jsonWriter.endArray();
            }
            if (started) {
                jsonWriter.endObject();
            }
        }
        if (fieldVariant.isFicheWith(FieldVariant.FICHE_SCORE)) {
            jsonWriter.key("score");
            int[] array = info.getScoreArray();
            int scoreLength = array.length;
            jsonWriter.array();
            for (int i = 0; i < scoreLength; i++) {
                jsonWriter.value(array[i]);
            }
            jsonWriter.endArray();
        }
    }

}
