/* SctServer - Copyright (c) 2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.json.motcledata;

import fr.exemole.sctserver.api.fieldvariant.FieldVariant;
import fr.exemole.sctserver.json.JsonUtils;
import java.io.IOException;
import java.util.Set;
import net.mapeadores.util.json.CommonJson;
import net.mapeadores.util.json.JSONWriter;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.text.Label;
import net.scrutari.data.MotcleData;
import net.scrutari.datauri.BaseURI;
import net.scrutari.datauri.MotcleURI;
import net.scrutari.searchengine.api.result.InLangOccurrences;
import net.scrutari.searchengine.api.result.MotcleSearchResultInfo;


/**
 *
 * @author Vincent Calame
 */
public class MotcleDataJson_B {

    private MotcleDataJson_B() {

    }

    public static void object(JSONWriter jsonWriter, MotcleData motcleData, FieldVariant fieldVariant, MotcleSearchResultInfo motcleSearchResultInfo, Lang lang) throws IOException {
        jsonWriter.object();
        properties(jsonWriter, motcleData, fieldVariant, motcleSearchResultInfo, lang);
        jsonWriter.endObject();
    }

    public static void properties(JSONWriter jsonWriter, MotcleData motcleData, FieldVariant fieldVariant, MotcleSearchResultInfo motcleSearchResultInfo, Lang lang) throws IOException {
        if (fieldVariant.isMotcleWith(FieldVariant.MOTCLE_CODEMOTCLE)) {
            jsonWriter.key("codemotcle");
            jsonWriter.value(motcleData.getCodeMotcle());
        }
        if (fieldVariant.isMotcleWith(FieldVariant.MOTCLE_CODETHESAURUS)) {
            jsonWriter.key("codethesaurus");
            jsonWriter.value(motcleData.getCodeThesaurus());
        }
        if (fieldVariant.isMotcleWith(FieldVariant.MOTCLE_CODEBASE)) {
            jsonWriter.key("codebase");
            jsonWriter.value(motcleData.getCodeBase());
        }
        if (fieldVariant.isMotcleWith(FieldVariant.MOTCLE_BASENAME)) {
            BaseURI baseURI = motcleData.getMotcleURI().getThesaurusURI().getBaseURI();
            jsonWriter.key("authority");
            jsonWriter.value(baseURI.getAuthority());
            jsonWriter.key("basename");
            jsonWriter.value(baseURI.getBaseName());
        }
        if (fieldVariant.isMotcleWith(FieldVariant.MOTCLE_THESAURUSNAME)) {
            MotcleURI motcleURI = motcleData.getMotcleURI();
            jsonWriter.key("thesaurusname");
            jsonWriter.value(motcleURI.getThesaurusURI().getThesaurusName());
        }
        if (fieldVariant.isMotcleWith(FieldVariant.MOTCLE_MOTCLEID)) {
            MotcleURI motcleURI = motcleData.getMotcleURI();
            jsonWriter.key("motcleid");
            jsonWriter.value(motcleURI.getMotcleId());
        }
        if (fieldVariant.isMotcleWith(FieldVariant.MOTCLE_LABELS)) {
            jsonWriter.key("labelMap");
            CommonJson.object(jsonWriter, motcleData);
        }
        if (fieldVariant.isMotcleWith(FieldVariant.MOTCLE_MLABELS)) {
            if (fieldVariant.getMotcleWithType(FieldVariant.MOTCLE_MLABELS) == FieldVariant.ALL_TYPE) {
                addDefaultMLabelArray(jsonWriter, motcleData, motcleSearchResultInfo, true);
            } else {
                addMLabelArray(jsonWriter, motcleData, motcleSearchResultInfo, lang);
            }
        }
        if (fieldVariant.isMotcleWith(FieldVariant.MOTCLE_ATTRIBUTES)) {
            jsonWriter.key("attrMap");
            CommonJson.object(jsonWriter, motcleData.getAttributHolder());
        }
    }

    private static void addDefaultMLabelArray(JSONWriter jsonWriter, MotcleData motcleData, MotcleSearchResultInfo motcleSearchResultInfo, boolean all) throws IOException {
        int labelCount = motcleData.getLabelCount();
        boolean inclus = false;
        for (int i = 0; i < labelCount; i++) {
            Label label = motcleData.getLabelByIndex(i);
            Lang labelLang = label.getLang();
            InLangOccurrences occurrences = motcleSearchResultInfo.getInLangOccurrenceListByLang(labelLang);
            if ((!all) && (occurrences == null)) {
                continue;
            }
            if (!inclus) {
                jsonWriter.key("mlabelArray");
                jsonWriter.array();
                inclus = true;
            }
            jsonWriter.object();
            jsonWriter.key("lang");
            jsonWriter.value(labelLang.toString());
            jsonWriter.key("mvalue");
            JsonUtils.writeMValue(jsonWriter, label.getLabelString(), occurrences.getSearchTokenOccurrenceList());
            jsonWriter.endObject();
        }
        if (inclus) {
            jsonWriter.endArray();
        }
    }

    private static void addMLabelArray(JSONWriter jsonWriter, MotcleData motcleData, MotcleSearchResultInfo motcleSearchResultInfo, Lang lang) throws IOException {
        InLangOccurrences uiOccurrences = motcleSearchResultInfo.getInLangOccurrenceListByLang(lang);
        if (uiOccurrences == null) {
            addDefaultMLabelArray(jsonWriter, motcleData, motcleSearchResultInfo, false);
            return;
        }
        jsonWriter.key("mlabelArray");
        jsonWriter.array();
        Label uiLib = motcleData.getLabelByLang(lang);
        jsonWriter.object();
        jsonWriter.key("lang");
        jsonWriter.value(lang.toString());
        jsonWriter.key("mvalue");
        JsonUtils.writeMValue(jsonWriter, uiLib.getLabelString(), uiOccurrences.getSearchTokenOccurrenceList());
        jsonWriter.endObject();
        int labelCount = motcleData.getLabelCount();
        if (labelCount > 1) {
            Set<Integer> operandeSet = JsonUtils.getOperandSet(uiOccurrences);
            for (int i = 0; i < labelCount; i++) {
                Label label = motcleData.getLabelByIndex(i);
                Lang labelLang = label.getLang();
                if (labelLang.equals(lang)) {
                    continue;
                }
                InLangOccurrences occurrences = motcleSearchResultInfo.getInLangOccurrenceListByLang(labelLang);
                if (occurrences == null) {
                    continue;
                }
                if (JsonUtils.isCovered(occurrences, operandeSet)) {
                    continue;
                }
                jsonWriter.object();
                jsonWriter.key("lang");
                jsonWriter.value(labelLang.toString());
                jsonWriter.key("mvalue");
                JsonUtils.writeMValue(jsonWriter, label.getLabelString(), occurrences.getSearchTokenOccurrenceList());
                jsonWriter.endObject();
            }
        }
        jsonWriter.endArray();
    }

}
