/* SctServer - Copyright (c) 2010-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.json;

import fr.exemole.sctserver.api.AttributeDefManager;
import fr.exemole.sctserver.api.ScrutariSession;
import fr.exemole.sctserver.api.fieldvariant.FieldVariant;
import fr.exemole.sctserver.json.fichedata.FicheDataJson_A;
import fr.exemole.sctserver.json.fichedata.FicheDataJson_B;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import net.mapeadores.util.json.JSONWriter;
import net.mapeadores.util.localisation.Lang;
import net.scrutari.data.FicheData;
import net.scrutari.db.api.AttributeKeyCoder;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.codes.Codes;
import net.scrutari.db.tools.util.BaseCheck;


/**
 *
 * @author Vincent Calame
 */
public class FicheJsonProducer extends AbstractJsonProducer {

    private final int version;
    private final ScrutariSession scrutariSession;
    private final Codes ficheCodes;
    private final FieldVariant fieldVariant;
    private final Lang lang;

    public FicheJsonProducer(int version, ScrutariSession scrutariSession, Codes ficheCodes, FieldVariant fieldVariant, Lang lang) {
        if (lang == null) {
            throw new IllegalArgumentException("lang is null");
        }
        this.version = version;
        this.scrutariSession = scrutariSession;
        this.ficheCodes = ficheCodes;
        this.fieldVariant = fieldVariant;
        this.lang = lang;
    }

    @Override
    public void writeJson(Appendable appendable) throws IOException {
        DataAccess dataAccess = scrutariSession.getScrutariDB().openDataAccess();
        JSONWriter jsonWriter = new JSONWriter(appendable);
        jsonWriter.object();
        if (version >= 3) {
            writeVersion3(jsonWriter, dataAccess);
        } else if (version == 2) {
            writeVersion2(jsonWriter, dataAccess);
        } else {
            writeVersion1(jsonWriter, dataAccess);
        }
        writeWarnings(jsonWriter);
        jsonWriter.endObject();
        dataAccess.close();
    }

    private void writeVersion3(JSONWriter jsonWriter, DataAccess dataAccess) throws IOException {
        AttributeDefManager attributeDefManager = scrutariSession.getEngine().getAttributeDefManager();
        AttributeKeyCoder attributeKeyCoder = scrutariSession.getScrutariDB().getAttributeKeyCoder();
        Map<Integer, BaseCheck> bufferMap = new HashMap<Integer, BaseCheck>();
        int ficheCount = ficheCodes.getCodeCount();
        jsonWriter.key("ficheArray");
        jsonWriter.array();
        for (int i = 0; i < ficheCount; i++) {
            int codefiche = ficheCodes.getCode(i);
            FicheData ficheData = dataAccess.getFicheDataByCode(codefiche);
            BaseCheck baseCheck = getBaseCheck(dataAccess, ficheData.getCodeBase(), bufferMap, lang);
            FicheDataJson_B.object(jsonWriter, ficheData, fieldVariant, baseCheck, null, attributeDefManager, attributeKeyCoder);
        }
        jsonWriter.endArray();
    }

    private void writeVersion2(JSONWriter jsonWriter, DataAccess dataAccess) throws IOException {
        AttributeDefManager attributeDefManager = scrutariSession.getEngine().getAttributeDefManager();
        AttributeKeyCoder attributeKeyCoder = scrutariSession.getScrutariDB().getAttributeKeyCoder();
        Map<Integer, BaseCheck> bufferMap = new HashMap<Integer, BaseCheck>();
        int ficheCount = ficheCodes.getCodeCount();
        jsonWriter.key("ficheArray");
        jsonWriter.array();
        for (int i = 0; i < ficheCount; i++) {
            int codefiche = ficheCodes.getCode(i);
            FicheData ficheData = dataAccess.getFicheDataByCode(codefiche);
            BaseCheck baseCheck = getBaseCheck(dataAccess, ficheData.getCodeBase(), bufferMap, lang);
            FicheDataJson_A.object(jsonWriter, ficheData, fieldVariant, baseCheck, null, attributeDefManager, attributeKeyCoder);
        }
        jsonWriter.endArray();
    }

    private void writeVersion1(JSONWriter jsonWriter, DataAccess dataAccess) throws IOException {
        AttributeDefManager attributeDefManager = scrutariSession.getEngine().getAttributeDefManager();
        AttributeKeyCoder attributeKeyCoder = scrutariSession.getScrutariDB().getAttributeKeyCoder();
        Map<Integer, BaseCheck> bufferMap = new HashMap<Integer, BaseCheck>();
        jsonWriter.key("ficheData");
        jsonWriter.object();
        int ficheCount = ficheCodes.getCodeCount();
        jsonWriter.key("ficheCount");
        jsonWriter.value(ficheCount);
        if (ficheCount > 0) {
            jsonWriter.key("ficheArray");
            jsonWriter.array();
            for (int i = 0; i < ficheCount; i++) {
                int codefiche = ficheCodes.getCode(i);
                FicheData ficheData = dataAccess.getFicheDataByCode(codefiche);
                BaseCheck baseCheck = getBaseCheck(dataAccess, ficheData.getCodeBase(), bufferMap, lang);
                FicheDataJson_A.object(jsonWriter, ficheData, fieldVariant, baseCheck, null, attributeDefManager, attributeKeyCoder);
            }
            jsonWriter.endArray();
        }
        jsonWriter.endObject();
    }

}
