/* SctServer - Copyright (c) 2012-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.json;

import fr.exemole.sctserver.api.fieldvariant.FieldVariant;
import fr.exemole.sctserver.json.motcledata.MotcleDataJson_A;
import fr.exemole.sctserver.json.motcledata.MotcleDataJson_B;
import java.io.IOException;
import net.mapeadores.util.json.JSONWriter;
import net.scrutari.data.MotcleData;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.ScrutariDB;
import net.scrutari.db.api.codes.Codes;


/**
 *
 * @author Vincent Calame
 */
public class MotcleJsonProducer extends AbstractJsonProducer {

    private final int version;
    private final ScrutariDB scrutariDB;
    private final Codes motcleCodes;
    private final FieldVariant fieldVariant;

    public MotcleJsonProducer(int version, ScrutariDB scrutariDB, Codes motcleCodes, FieldVariant fieldVariant) {
        this.version = version;
        this.scrutariDB = scrutariDB;
        this.motcleCodes = motcleCodes;
        this.fieldVariant = fieldVariant;
    }

    @Override
    public void writeJson(Appendable appendable) throws IOException {
        DataAccess dataAccess = scrutariDB.openDataAccess();
        JSONWriter jsonWriter = new JSONWriter(appendable);
        jsonWriter.object();
        if (version >= 3) {
            writeVersion3(jsonWriter, dataAccess);
        } else if (version == 2) {
            writeVersion2(jsonWriter, dataAccess);
        } else {
            writeVersion1(jsonWriter, dataAccess);
        }
        writeWarnings(jsonWriter);
        jsonWriter.endObject();
        dataAccess.close();
    }

    private void writeVersion3(JSONWriter jsonWriter, DataAccess dataAccess) throws IOException {
        int motcleCount = motcleCodes.getCodeCount();
        jsonWriter.key("motcleArray");
        jsonWriter.array();
        for (int i = 0; i < motcleCount; i++) {
            int codemotcle = motcleCodes.getCode(i);
            MotcleData motcleData = dataAccess.getMotcleDataByCode(codemotcle);
            MotcleDataJson_B.object(jsonWriter, motcleData, fieldVariant, null, null);
        }
        jsonWriter.endArray();
    }

    private void writeVersion2(JSONWriter jsonWriter, DataAccess dataAccess) throws IOException {
        int motcleCount = motcleCodes.getCodeCount();
        jsonWriter.key("motcleArray");
        jsonWriter.array();
        for (int i = 0; i < motcleCount; i++) {
            int codemotcle = motcleCodes.getCode(i);
            MotcleData motcleData = dataAccess.getMotcleDataByCode(codemotcle);
            MotcleDataJson_A.object(jsonWriter, motcleData, fieldVariant, null, null);
        }
        jsonWriter.endArray();
    }

    private void writeVersion1(JSONWriter jsonWriter, DataAccess dataAccess) throws IOException {
        jsonWriter.key("motcleData");
        jsonWriter.object();
        int motcleCount = motcleCodes.getCodeCount();
        jsonWriter.key("motcleCount");
        jsonWriter.value(motcleCount);
        if (motcleCount > 0) {
            jsonWriter.key("motcleArray");
            jsonWriter.array();
            for (int i = 0; i < motcleCount; i++) {
                int codemotcle = motcleCodes.getCode(i);
                MotcleData motcleData = dataAccess.getMotcleDataByCode(codemotcle);
                MotcleDataJson_A.object(jsonWriter, motcleData, fieldVariant, null, null);
            }
            jsonWriter.endArray();
        }
        jsonWriter.endObject();
    }

}
