/* SctServer - Copyright (c) 2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.tools.conf.dom;

import fr.exemole.sctserver.tools.DataValidatorBuilder;
import fr.exemole.sctserver.tools.ErrorUtils;
import java.text.ParseException;
import net.mapeadores.util.attr.AttributeKey;
import net.mapeadores.util.logging.MessageHandler;
import net.mapeadores.util.xml.DOMUtils;
import net.mapeadores.util.xml.ElementHandler;
import net.scrutari.data.DataValidator;
import net.scrutari.data.ScrutariAttributes;
import org.w3c.dom.Element;


/**
 *
 * @author Vincent Calame
 */
public class ValidationDOMReader {

    private final DataValidatorBuilder dataValidatorBuilder;
    private final MessageHandler messageHandler;

    public ValidationDOMReader(DataValidatorBuilder dataValidatorBuilder, MessageHandler messageHandler) {
        this.dataValidatorBuilder = dataValidatorBuilder;
        this.messageHandler = messageHandler;
    }

    public void readValidation(Element element, String xpath) {
        xpath = xpath + "/" + element.getTagName();
        DOMUtils.readChildren(element, new RootHandler(xpath));

    }


    private class RootHandler implements ElementHandler {

        private final String rootXpath;

        private RootHandler(String xpath) {
            this.rootXpath = xpath;
        }

        @Override
        public void readElement(Element element) {
            String tagName = element.getTagName();
            String xpath = rootXpath + "/" + tagName;
            switch (tagName) {
                case "mandatory":
                    try {
                        short field = getField(element, xpath);
                        dataValidatorBuilder.setMandatory(field);
                    } catch (IllegalArgumentException iae) {

                    }
                    break;
                case "check":
                    try {
                        short field = getField(element, xpath);
                        int min = getInt(element, xpath, "min");
                        int max = getInt(element, xpath, "max");
                        dataValidatorBuilder.setCheck(field, min, max);
                    } catch (IllegalArgumentException iae) {

                    }
                    break;
                case "unique":
                    String nameSpace = element.getAttribute("ns").trim();
                    String localKey = element.getAttribute("key").trim();
                    if (nameSpace.length() == 0) {
                        ErrorUtils.severe(messageHandler, "_ error.empty.xml.attribute", xpath, "ns");
                    } else if (localKey.length() == 0) {
                        ErrorUtils.severe(messageHandler, "_ error.empty.xml.attribute", xpath, "key");
                    } else {
                        try {
                            AttributeKey attributeKey = ScrutariAttributes.toAttributeKey(nameSpace, localKey);
                            dataValidatorBuilder.addUniqueValue(attributeKey);
                        } catch (ParseException pe) {
                            ErrorUtils.severe(messageHandler, "_ error.wrong.xml.attributevalue", xpath + "[@ns='" + nameSpace + "' and @key='" + localKey + "']", "key|ns", nameSpace + ":" + localKey);
                        }
                    }
                    break;
                default:
                    ErrorUtils.warning(messageHandler, "_ error.unknown.xml.tag", xpath);
            }

        }

    }

    private short getField(Element element, String xpath) {
        String field = element.getAttribute("field");
        if (field.length() == 0) {
            ErrorUtils.severe(messageHandler, "_ error.empty.xml.attribute", xpath, "name");
            throw new IllegalArgumentException();
        }
        switch (field) {
            case "geoloc":
                return DataValidator.GEOLOC_FIELD;
            case "soustitre":
                return DataValidator.SOUSTITRE_FIELD;
            default:
                ErrorUtils.warning(messageHandler, "_ error.unknown.xml.attributevalue", xpath, "field", field);
                throw new IllegalArgumentException();
        }
    }

    private int getInt(Element element, String xpath, String attributeName) {
        String intValue = element.getAttribute(attributeName);
        if (intValue.length() == 0) {
            ErrorUtils.severe(messageHandler, "_ error.empty.xml.attribute", xpath, attributeName);
            throw new IllegalArgumentException();
        }
        try {
            int value = Integer.parseInt(intValue);
            return value;
        } catch (NumberFormatException nfe) {
            ErrorUtils.severe(messageHandler, "error.wrong.xml.attributevalue_integer", xpath, attributeName, intValue);
            throw new IllegalArgumentException();
        }
    }

}
