/* SctServer - Copyright (c) 2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.tools.conf.factory;

import fr.exemole.sctserver.api.fieldvariant.FieldVariant;
import fr.exemole.sctserver.tools.ErrorUtils;
import fr.exemole.sctserver.tools.conf.dom.FieldsDOMReader;
import java.io.File;
import java.io.IOException;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import net.mapeadores.util.logging.CategorizedMessageEntry;
import net.mapeadores.util.logging.SimpleMessageHandler;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;


/**
 *
 * @author Vincent Calame
 */
class FieldsConf {

    private String processURI;
    private List<CategorizedMessageEntry> messageList;
    private Map<String, FieldVariant> map;

    private FieldsConf() {

    }

    Map<String, FieldVariant> getFieldVariantMap() {
        return map;
    }

    static FieldsConf load(File f) throws IOException, ParserConfigurationException {
        FieldsConf fieldsConf = new FieldsConf();
        fieldsConf.init(f);
        return fieldsConf;
    }

    String getProcessURI() {
        return processURI;
    }

    List<CategorizedMessageEntry> getMessageList() {
        return messageList;
    }

    private void init(File f) throws IOException, ParserConfigurationException {
        processURI = f.getPath();
        SimpleMessageHandler simpleHandler = new SimpleMessageHandler();
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        Document document;
        try {
            DocumentBuilder db = dbf.newDocumentBuilder();
            document = db.parse(f);
        } catch (SAXException saxe) {
            ErrorUtils.xml(simpleHandler, "_ error.exception.xml.sax", saxe.getMessage());
            messageList = simpleHandler.toList();
            return;
        }
        map = new LinkedHashMap<String, FieldVariant>();
        FieldsDOMReader domReader = new FieldsDOMReader(map, simpleHandler);
        domReader.readFields(document.getDocumentElement(), "");
        messageList = simpleHandler.toList();
    }

}
