/* SctServer - Copyright (c) 2013-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.tools.feed;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import net.mapeadores.util.localisation.Lang;
import net.scrutari.data.FicheData;
import net.scrutari.db.api.ScrutariDBName;
import net.scrutari.db.api.codes.Codes;
import net.scrutari.db.api.codes.ElementCodes;
import net.scrutari.searchengine.api.result.FicheSearchResult;
import net.scrutari.searchengine.api.result.FicheSearchResultGroup;
import net.scrutari.searchengine.api.result.FicheSearchResultInfo;
import net.scrutari.searchengine.tools.EligibilityBuffer;


/**
 *
 * @author Vincent Calame
 */
public class FicheFeedCodes implements ElementCodes {

    private final Lang lang;
    private final List<FicheWrapper> ficheWrapperList = new ArrayList<FicheWrapper>();

    public FicheFeedCodes(Lang lang) {
        this.lang = lang;
    }

    public int getFicheCount() {
        return ficheWrapperList.size();
    }

    public FicheWrapper getFicheWrapper(int i) {
        return ficheWrapperList.get(i);
    }

    public Lang getLang() {
        return lang;
    }

    @Override
    public int getCodeCount() {
        return ficheWrapperList.size();
    }

    @Override
    public int getCode(int i) {
        return ficheWrapperList.get(i).getFicheData().getCodeFiche();
    }

    @Override
    public int getSubsetCode(int i) {
        return ficheWrapperList.get(i).getCorpusWrapper().getCorpusData().getCodeCorpus();
    }

    void addFicheData(FicheData ficheData, CorpusWrapper corpusWrapper, BaseWrapper baseWrapper, ScrutariDBName scrutariDBName) {
        ficheWrapperList.add(FicheWrapper.wrapFiche(ficheData, corpusWrapper, baseWrapper, scrutariDBName));
    }

    void update(FicheSearchResult ficheSearchResult) {
        Set<Integer> codeSet = new HashSet<Integer>();
        for (FicheSearchResultGroup ficheSearchResultGroup : ficheSearchResult.getFicheSearchResultGroupList()) {
            for (FicheSearchResultInfo info : ficheSearchResultGroup.getFicheSearchResultInfoList()) {
                codeSet.add(info.getCode());
            }
        }
        for (Iterator<FicheWrapper> it = ficheWrapperList.iterator(); it.hasNext();) {
            FicheWrapper ficheWrapper = it.next();
            if (!codeSet.contains(ficheWrapper.getFicheData().getCodeFiche())) {
                it.remove();
            }
        }
    }

    void update(EligibilityBuffer eligibilityBuffer, List<Lang> langList, Codes corpusCodes) {
        Set<Lang> langSet = null;
        if (!langList.isEmpty()) {
            langSet = new HashSet<Lang>(langList);
        }
        Set<Integer> corpusSet = null;
        if (corpusCodes != null) {
            corpusSet = new HashSet<Integer>();
            int count = corpusCodes.getCodeCount();
            for (int i = 0; i < count; i++) {
                corpusSet.add(corpusCodes.getCode(i));
            }
        }
        for (Iterator<FicheWrapper> it = ficheWrapperList.iterator(); it.hasNext();) {
            FicheWrapper ficheWrapper = it.next();
            boolean remove = false;
            FicheData ficheData = ficheWrapper.getFicheData();
            if (langSet != null) {
                if ((!remove) && (!langSet.contains(ficheData.getLang()))) {
                    remove = true;
                }
            }
            if (corpusSet != null) {
                if ((!remove) && (!corpusSet.contains(ficheData.getCodeCorpus()))) {
                    remove = true;
                }
            }
            if (eligibilityBuffer != null) {
                if ((!remove) && (!eligibilityBuffer.acceptFiche(ficheData.getCodeFiche()))) {
                    remove = true;
                }
            }
            if (remove) {
                it.remove();
            }
        }
    }

}
