/* SctServer - Copyright (c) 2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.tools.fieldvariant;

import fr.exemole.sctserver.api.fieldvariant.FieldVariant;
import java.text.ParseException;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.util.attr.AttributeKey;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class FieldVariantBuilder {

    private final String name;
    private final boolean[] ficheWithArray = new boolean[22];
    private final short[] ficheWithTypeArray = new short[22];
    private final boolean[] motcleWithArray = new boolean[9];
    private final short[] motcleWithTypeArray = new short[9];
    private final Map<String, AliasBuilder> ficheAliasBuilderMap = new LinkedHashMap<String, AliasBuilder>();

    public FieldVariantBuilder(String name) {
        if (!name.isEmpty()) {
            if (!StringUtils.isTechnicalName(name, true)) {
                throw new IllegalArgumentException("wrong name: " + name);
            }
        }
        this.name = name;
        ficheWithArray[FieldVariant.FICHE_CODEFICHE] = true;
        motcleWithArray[FieldVariant.MOTCLE_CODEMOTCLE] = true;
    }

    public void setFicheWith(int field, boolean value) {
        if ((field < 0) || (field >= ficheWithArray.length)) {
            return;
        }
        if (value) {
            ficheWithArray[field] = true;
            ficheWithTypeArray[field] = FieldVariant.CONCERNED_TYPE;
        } else {
            ficheWithArray[field] = false;
            ficheWithTypeArray[field] = FieldVariant.NONE_TYPE;
        }
    }

    public void setFicheWithType(int field, short value) {
        if ((field < 0) || (field >= ficheWithTypeArray.length)) {
            return;
        }
        ficheWithTypeArray[field] = value;
        ficheWithArray[field] = (value != FieldVariant.NONE_TYPE);
    }

    public void setMotcleWith(int field, boolean value) {
        if ((field < 0) || (field >= motcleWithArray.length)) {
            return;
        }
        if (value) {
            motcleWithArray[field] = true;
            motcleWithTypeArray[field] = FieldVariant.CONCERNED_TYPE;
        } else {
            motcleWithArray[field] = false;
            motcleWithTypeArray[field] = FieldVariant.NONE_TYPE;
        }
    }

    public void setMotcleWithType(int field, short value) {
        if ((field < 0) || (field >= motcleWithTypeArray.length)) {
            return;
        }
        motcleWithTypeArray[field] = value;
        motcleWithArray[field] = (value != FieldVariant.NONE_TYPE);
    }

    public void parseFicheFields(String ficheFields) {
        String[] tokens = StringUtils.getTechnicalTokens(ficheFields, true);
        for (String token : tokens) {
            String constantToken = token.toLowerCase();
            boolean other = false;
            switch (constantToken) {
                case "-codefiche":
                    setFicheWith(FieldVariant.FICHE_CODEFICHE, false);
                    break;
                case "codecorpus":
                    setFicheWith(FieldVariant.FICHE_CODECORPUS, true);
                    break;
                case "codebase":
                    setFicheWith(FieldVariant.FICHE_CODEBASE, true);
                    break;
                case "basename":
                    setFicheWith(FieldVariant.FICHE_BASENAME, true);
                    break;
                case "corpusname":
                    setFicheWith(FieldVariant.FICHE_CORPUSNAME, true);
                    break;
                case "ficheid":
                    setFicheWith(FieldVariant.FICHE_FICHEID, true);
                    break;
                case "titre":
                    setFicheWith(FieldVariant.FICHE_TITRE, true);
                    break;
                case "soustitre":
                    setFicheWith(FieldVariant.FICHE_SOUSTITRE, true);
                    break;
                case "href":
                    setFicheWith(FieldVariant.FICHE_HREF, true);
                    break;
                case "lang":
                    setFicheWith(FieldVariant.FICHE_LANG, true);
                    break;
                case "year":
                case "annee":
                    setFicheWith(FieldVariant.FICHE_YEAR, true);
                    break;
                case "date":
                    setFicheWith(FieldVariant.FICHE_DATE, true);
                    break;
                case "ficheicon":
                    setFicheWith(FieldVariant.FICHE_FICHEICON, true);
                    break;
                case "icon":
                    setFicheWith(FieldVariant.FICHE_ICON, true);
                    break;
                case "geo":
                    setFicheWith(FieldVariant.FICHE_GEO, true);
                    break;
                case "complements":
                    setFicheWith(FieldVariant.FICHE_COMPLEMENTS, true);
                    break;
                case "attrs":
                    setFicheWithType(FieldVariant.FICHE_ATTRIBUTES, FieldVariant.CONCERNED_TYPE);
                    break;
                case "attrs_primary":
                    setFicheWithType(FieldVariant.FICHE_ATTRIBUTES, FieldVariant.PRIMARY_TYPE);
                    break;
                case "attrs_all":
                    setFicheWithType(FieldVariant.FICHE_ATTRIBUTES, FieldVariant.ALL_TYPE);
                    break;
                case "mtitre":
                case "mtitre_def":
                    setFicheWith(FieldVariant.FICHE_MTITRE, true);
                    break;
                case "msoustitre":
                case "msoustitre_def":
                    setFicheWith(FieldVariant.FICHE_MSOUSTITRE, true);
                    break;
                case "mcomplements":
                    setFicheWithType(FieldVariant.FICHE_MCOMPLEMENTS, FieldVariant.CONCERNED_TYPE);
                    break;
                case "mcomplements_all":
                    setFicheWithType(FieldVariant.FICHE_MCOMPLEMENTS, FieldVariant.ALL_TYPE);
                    break;
                case "mattrs":
                    setFicheWithType(FieldVariant.FICHE_MATTRIBUTES, FieldVariant.CONCERNED_TYPE);
                    break;
                case "mattrs_primary":
                    setFicheWithType(FieldVariant.FICHE_MATTRIBUTES, FieldVariant.PRIMARY_TYPE);
                    break;
                case "mattrs_all":
                    setFicheWithType(FieldVariant.FICHE_MATTRIBUTES, FieldVariant.ALL_TYPE);
                    break;
                case "score":
                    setFicheWith(FieldVariant.FICHE_SCORE, true);
                    break;
                default:
                    other = true;
            }
            if (other) {
                try {
                    AttributeKey attributeKey = AttributeKey.parse(token);
                    if (!ficheAliasBuilderMap.containsKey(token)) {
                        AliasBuilder builder = new AliasBuilder(token);
                        builder.addAttributeField(attributeKey);
                        ficheAliasBuilderMap.put(token, builder);
                    }
                } catch (ParseException pe) {

                }
            }

        }
    }

    public void parseMotcleFields(String motcleFields) {
        String[] tokens = StringUtils.getTechnicalTokens(motcleFields, true);
        for (String token : tokens) {
            String constantToken = token.toLowerCase();
            switch (constantToken) {
                case "-codemotcle":
                    setMotcleWith(FieldVariant.MOTCLE_CODEMOTCLE, false);
                    break;
                case "codethesaurus":
                    setMotcleWith(FieldVariant.MOTCLE_CODETHESAURUS, true);
                    break;
                case "codebase":
                    setMotcleWith(FieldVariant.MOTCLE_CODEBASE, true);
                    break;
                case "basename":
                    setMotcleWith(FieldVariant.MOTCLE_BASENAME, true);
                    break;
                case "thesaurusname":
                    setMotcleWith(FieldVariant.MOTCLE_THESAURUSNAME, true);
                    break;
                case "motcleid":
                    setMotcleWith(FieldVariant.MOTCLE_MOTCLEID, true);
                    break;
                case "labels":
                case "libelles":
                    setMotcleWith(FieldVariant.MOTCLE_LABELS, true);
                    break;
                case "attrs":
                    setMotcleWith(FieldVariant.MOTCLE_ATTRIBUTES, true);
                    break;
                case "mlabels":
                case "mlibelles":
                    setMotcleWithType(FieldVariant.MOTCLE_MLABELS, FieldVariant.CONCERNED_TYPE);
                    break;
                case "mlabels_all":
                case "mlabels_def":
                case "mlibelles_def":
                    setMotcleWithType(FieldVariant.MOTCLE_MLABELS, FieldVariant.ALL_TYPE);
                    break;

            }
        }
    }

    public AliasBuilder getAliasBuilder(String aliasName) {
        if ((aliasName == null) || (aliasName.length() == 0)) {
            throw new IllegalArgumentException("aliasName is null or empty");
        }
        AliasBuilder builder = ficheAliasBuilderMap.get(aliasName);
        if (builder == null) {
            builder = new AliasBuilder(aliasName);
            ficheAliasBuilderMap.put(aliasName, builder);
        }
        return builder;
    }

    public FieldVariant toFieldVariant() {
        int ficheLength = ficheWithArray.length;
        boolean[] finalFicheWithArray = new boolean[ficheLength];
        System.arraycopy(ficheWithArray, 0, finalFicheWithArray, 0, ficheLength);
        short[] finalFicheWithTypeArray = new short[ficheLength];
        System.arraycopy(ficheWithTypeArray, 0, finalFicheWithTypeArray, 0, ficheLength);
        int motcleLength = motcleWithArray.length;
        boolean[] finalMotcleWithArray = new boolean[motcleLength];
        System.arraycopy(motcleWithArray, 0, finalMotcleWithArray, 0, motcleLength);
        short[] finalMotcleWithTypeArray = new short[motcleLength];
        System.arraycopy(motcleWithTypeArray, 0, finalMotcleWithTypeArray, 0, motcleLength);
        List<FieldVariant.Alias> ficheAliasList = toList(ficheAliasBuilderMap);
        return new InternalFieldVariant(name, finalFicheWithArray, finalFicheWithTypeArray, finalMotcleWithArray, finalMotcleWithTypeArray, ficheAliasList);
    }

    private List<FieldVariant.Alias> toList(Map<String, AliasBuilder> map) {
        int size = map.size();
        if (size == 0) {
            return FieldVariantUtils.EMPTY_ALIASLIST;
        }
        FieldVariant.Alias[] array = new FieldVariant.Alias[size];
        int p = 0;
        for (AliasBuilder builder : map.values()) {
            array[p] = builder.toAlias();
            p++;
        }
        return FieldVariantUtils.wrap(array);
    }


    private static class InternalFieldVariant implements FieldVariant {

        private final String name;
        private final boolean[] ficheWithArray;
        private final short[] ficheWithTypeArray;
        private final boolean[] motcleWithArray;
        private final short[] motcleWithTypeArray;
        private final List<FieldVariant.Alias> ficheAliasList;

        private InternalFieldVariant(String name, boolean[] ficheWithArray, short[] ficheWithTypeArray, boolean[] motcleWithArray, short[] motcleWithTypeArray, List<FieldVariant.Alias> ficheAliasList) {
            this.name = name;
            this.ficheWithArray = ficheWithArray;
            this.ficheWithTypeArray = ficheWithTypeArray;
            this.motcleWithArray = motcleWithArray;
            this.motcleWithTypeArray = motcleWithTypeArray;
            this.ficheAliasList = ficheAliasList;
        }

        @Override
        public String getName() {
            return name;
        }

        @Override
        public boolean isFicheWith(int field) {
            try {
                return ficheWithArray[field];
            } catch (IndexOutOfBoundsException e) {
                return false;
            }
        }

        @Override
        public short getFicheWithType(int field) {
            try {
                return ficheWithTypeArray[field];
            } catch (IndexOutOfBoundsException e) {
                return FieldVariant.NONE_TYPE;
            }
        }

        @Override
        public boolean isMotcleWith(int field) {
            try {
                return motcleWithArray[field];
            } catch (IndexOutOfBoundsException e) {
                return false;
            }
        }

        @Override
        public short getMotcleWithType(int field) {
            try {
                return motcleWithTypeArray[field];
            } catch (IndexOutOfBoundsException e) {
                return FieldVariant.NONE_TYPE;
            }
        }

        @Override
        public List<Alias> getFicheAliasList() {
            return ficheAliasList;
        }

    }


}
