/* SctEngine - Copyright (c) 2012-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.xml.feed;

import fr.exemole.sctserver.api.ScrutariSession;
import fr.exemole.sctserver.api.SctEngine;
import fr.exemole.sctserver.api.feed.SctFeedInfo;
import fr.exemole.sctserver.api.feed.SctFeedMetadata;
import fr.exemole.sctserver.api.feed.SctFeedOptions;
import java.io.IOException;
import net.mapeadores.util.text.Label;
import net.mapeadores.util.text.LabelHolder;
import net.mapeadores.util.text.LabelUtils;
import net.mapeadores.util.xml.AppendableXMLPart;
import net.mapeadores.util.xml.AppendableXMLWriter;
import net.mapeadores.util.xml.XMLUtils;
import net.mapeadores.util.xml.ns.NameSpace;
import net.scrutari.db.api.DataAccess;


/**
 *
 * @author Vincent Calame
 */
public class AtomFeedXMLPart extends AppendableXMLPart {

    private final ScrutariSession scrutariSession;
    private final SctFeedInfo sctFeedInfo;
    private final String feedId;
    private final SctFeedOptions feedOptions;

    public AtomFeedXMLPart(AppendableXMLWriter xmlWriter, SctEngine engine, ScrutariSession scrutariSession, SctFeedOptions feedOptions) {
        super(xmlWriter);
        this.scrutariSession = scrutariSession;
        this.feedOptions = feedOptions;
        this.feedId = initFeedId(engine);
        this.sctFeedInfo = scrutariSession.getSctFeedInfo();
    }

    public void appendFeed() throws IOException {
        DataAccess dataAccess = scrutariSession.getScrutariDB().openDataAccess();
        startOpenTag("feed");
        XMLUtils.appendNameSpaceAttribute(this, NameSpace.ATOM_NAMESPACE);
        endOpenTag();
        addSimpleElement("title", getTitle());
        addSimpleElement("updated", sctFeedInfo.getLastScrutariDBNameList().get(0).toGmtIsoFormat());
        openTag("author");
        addSimpleElement("name", getAuthorName());
        closeTag("author");
        addSimpleElement("id", feedId);
        AbstractEntriesXMLPart entriesXMLPart = AbstractEntriesXMLPart.newInstance(feedOptions.getSctFeedType(), this, scrutariSession);
        entriesXMLPart.appendEntries(feedId, sctFeedInfo, dataAccess, feedOptions);
        closeTag("feed");
        dataAccess.close();
    }

    private String getTitle() {
        String title = null;
        SctFeedMetadata metadata = sctFeedInfo.getMetadata();
        if (metadata != null) {
            LabelHolder titleLabelHolder = metadata.getTitle();
            if (titleLabelHolder != null) {
                Label label = LabelUtils.getLangPartCheckedLabel(titleLabelHolder, feedOptions.getLang());
                if (label != null) {
                    title = label.getLabelString();
                }
            }
        }
        if (title == null) {
            title = "Scrutari " + scrutariSession.getEngineName();
        }
        return title;
    }

    private String getAuthorName() {
        String authorName = null;
        SctFeedMetadata metadata = sctFeedInfo.getMetadata();
        if (metadata != null) {
            LabelHolder authorNameLabelHolder = metadata.getAuthorName();
            if (authorNameLabelHolder != null) {
                Label label = LabelUtils.getLangPartCheckedLabel(authorNameLabelHolder, feedOptions.getLang());
                if (label != null) {
                    authorName = label.getLabelString();
                }
            }
        }
        if (authorName == null) {
            authorName = scrutariSession.getEngineName();
        }
        return authorName;
    }

    private String initFeedId(SctEngine engine) {
        StringBuilder buf = new StringBuilder();
        buf.append("scrutarifeed:");
        buf.append("/");
        buf.append((String) engine.getEngineContext().getWebappName());
        buf.append("/");
        buf.append(engine.getEngineName());
        return buf.toString();
    }

}
