/* UtilLib - Copyright (c) 2011-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.attr;

import java.util.HashMap;
import java.util.Map;
import net.mapeadores.util.misc.DefBuilder;
import net.mapeadores.util.text.LabelHolder;
import net.mapeadores.util.text.Phrases;
import net.mapeadores.util.text.PhrasesBuilder;


/**
 *
 * @author Vincent Calame
 */
public class AttributeDefBuilder extends DefBuilder {

    private final AttributeKey attributeKey;
    private final PhrasesBuilder phrasesBuilder = new PhrasesBuilder();
    private final Map<String, Object> defObjectMap = new HashMap<String, Object>();

    public AttributeDefBuilder(AttributeKey attributeKey) {
        if (attributeKey == null) {
            throw new IllegalArgumentException("attributeKey is null");
        }
        this.attributeKey = attributeKey;
    }

    public AttributeKey getAttributeKey() {
        return attributeKey;
    }

    public PhrasesBuilder getPhrasesBuilder() {
        return phrasesBuilder;
    }

    public void putDefObject(String objectName, Object object) {
        defObjectMap.put(objectName, object);
    }

    public AttributeDef toAttributeDef() {
        LabelHolder labelHolder = toLabelHolder();
        AttributeHolder attributeHolder = toAttributeHolder();
        Map<String, Object> finalDefObjectMap = new HashMap<String, Object>(defObjectMap);
        Phrases phrases = phrasesBuilder.toPhrases();
        return new InternalAttributeDef(attributeKey, finalDefObjectMap, phrases, labelHolder, attributeHolder);
    }


    private static class InternalAttributeDef extends DefBuilder.Def implements AttributeDef {

        private final AttributeKey attributeKey;
        private final Map<String, Object> defObjectMap;
        private final Phrases phrases;

        private InternalAttributeDef(AttributeKey attributeKey, Map<String, Object> defObjectMap, Phrases phrases,
                LabelHolder labelHolder, AttributeHolder attributeHolder) {
            super(labelHolder, attributeHolder);
            this.attributeKey = attributeKey;
            this.defObjectMap = defObjectMap;
            this.phrases = phrases;
        }

        @Override
        public AttributeKey getAttributeKey() {
            return attributeKey;
        }

        @Override
        public Object getDefObject(String objectName) {
            return defObjectMap.get(objectName);
        }

        @Override
        public Phrases getPhrases() {
            return phrases;
        }

    }

}
