/* UtilLib - Copyright (c) 2016-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.conditions;

import java.util.ArrayList;
import java.util.List;


/**
 *
 * @author Vincent Calame
 */
public class ConditionBuilder {

    private final short mode;
    private final List<TextTest> excludingList = new ArrayList<TextTest>();
    private final List<TextTest> includingList = new ArrayList<TextTest>();

    public ConditionBuilder(short mode) {
        this.mode = mode;
    }

    public void addTextTest(TextTest textTest) {
        short testType = textTest.getTestType();
        if (ConditionsUtils.isExcludingPartTestType(testType)) {
            if (excludingList.size() < 31) {
                excludingList.add(ConditionsUtils.cloneTextTest(textTest));
            }
        } else if (ConditionsUtils.isIncludingPartTestType(testType)) {
            if (includingList.size() < 31) {
                includingList.add(ConditionsUtils.cloneTextTest(textTest));
            }
        }
    }

    public Condition toCondition() {
        TextTest[] excludingArray = excludingList.toArray(new TextTest[excludingList.size()]);
        TextTest[] includingArray = includingList.toArray(new TextTest[includingList.size()]);
        return new InternalCondition(mode, ConditionsUtils.wrap(excludingArray), ConditionsUtils.wrap(includingArray));
    }

    public static Condition build(TextTest textTest) {
        ConditionBuilder conditionBuilder = new ConditionBuilder(ConditionsConstants.MODE_AND);
        conditionBuilder.addTextTest(textTest);
        return conditionBuilder.toCondition();
    }


    private static class InternalCondition implements Condition {

        private final short mode;
        private final List<TextTest> excludingList;
        private final List<TextTest> includingList;

        public InternalCondition(short mode, List<TextTest> excludingList, List<TextTest> includingList) {
            this.mode = mode;
            this.excludingList = excludingList;
            this.includingList = includingList;
        }

        @Override
        public short getMode() {
            return mode;
        }

        @Override
        public List<TextTest> getExcludingTextTestList() {
            return excludingList;
        }

        @Override
        public List<TextTest> getIncludingTextTestList() {
            return includingList;
        }


    }

}
